@@ -89,7 +89,9 @@
 
 #if HAVE_DIRENT_H
 # include <dirent.h>
+# ifndef NAMLEN
 # define NAMLEN(dirent) strlen((dirent)->d_name)
+# endif
 #else
 # define dirent direct
 # define NAMLEN(dirent) (dirent)->d_namlen
@@ -134,6 +136,8 @@
 # define MAXNAMLEN 512				/* for all other Unix */
 #endif
 
+#define BASENAMELEN		(MAXNAMLEN - 5)
+
 #ifdef HAVE_ERRNO_H
 # include <errno.h>
 #endif
@@ -205,13 +209,15 @@
 #endif
 
 #ifdef OS2
-#define TMPNAME1		"$TMP/viXXXXXX"
-#define TMPNAME2		"$TMP/voXXXXXX"
-#define TMPNAMELEN		128
+/*
+ * Try several directories to put the temp files.
+ */
+#define TEMPDIRNAMES	"$TMP", "$TEMP", "c:\\TMP", "c:\\TEMP", ""
+#define TEMPNAME		"v?XXXXXX"
+#define TEMPNAMELEN		128
 #else
-#define TMPNAME1		"/tmp/viXXXXXX"
-#define TMPNAME2		"/tmp/voXXXXXX"
-#define TMPNAMELEN		15
+#define TEMPNAME		"/tmp/v?XXXXXX"
+#define TEMPNAMELEN		15
 #endif
 
 /*
@@ -231,8 +237,6 @@
 # define MAXMEMTOT		2048		/* use up to 2048Kbyte for Vim */
 #endif
 
-#define BASENAMELEN		(MAXNAMLEN - 5)
-
 /* memmove is not present on all systems, use memmove, bcopy, memcpy or our
  * own version */
 /* Some systems have (void *) arguments, some (char *). If we use (char *) it
